var mongoose = require('mongoose');
var Schema   = mongoose.Schema;
var crypto   = require('crypto');
var jwt      = require('jsonwebtoken');

var userSchema = new Schema({
    username: { type: String, lowercase: true},
    name: String,
    city: String,
    state: String,
    hash: String,
    salt: String,
    borrowRequests: [String],
    lendRequests: [String],
    borrowed: [String],
    lent: [String]
});

userSchema.methods.setPassword = function(password){
    this.salt = crypto.randomBytes(16).toString('hex');
    this.hash = crypto.pbkdf2Sync(password, this.salt, 1000, 64).toString('hex');
};

userSchema.methods.validPassword = function(password) {
    var hash = crypto.pbkdf2Sync(password, this.salt, 1000, 64).toString('hex');
    return this.hash === hash;
};

userSchema.methods.generateJWT = function() {
    var today = new Date();
    var exp = new Date(today);
    exp.setDate(today.getDate() + 60);

    return jwt.sign({
        _id: this._id,
        username: this.username,
        exp: parseInt(exp.getTime() / 1000),
    }, 'SECRET');
};

var User = mongoose.model('User', userSchema);

module.exports = {
    user: User
}